@@ -1,5 +1,5 @@
 testthat::context("Test MSE Plotting functions")
-windows()
+dev.new()       
 DLMextra()
 library(DLMextra)
 rm(list=ls())
@@ -19,11 +19,8 @@ for (ff in funs) {
       } else {
         testthat::expect_error(fun(MSEobj), NA, info=ff) 
       }
-       
     }
-    
   })
 }
 
-
-
+if(!is.null(dev.list()))  dev.off()
